
import json
import requests


class Openapi(object):
    """
    开放接口，用于发送消息。
    """
    baseUrl = "https://chat-go.jwzhd.com/open-apis/v1"

    def __init__(self, token: str) -> None:
        self.token = token

    def sendMessage(self, recvId: str, recvType: str, content: map):
        """
        单条，发送文本消息
        """
        return self.sendMessage(recvId, recvType, "text", content)

    def sendMarkdownMessage(self, recvId: str, recvType: str, content: map):
        """
        单条，发送markdown消息
        """
        return self.sendMessage(recvId, recvType, "markdown", content)
    
    def sendMessage(self, recvId: str, recvType: str, contentType: str, content: map):
        """
        单条，发送单条消息
        """
        params = {
            "recvId": recvId, 
            "recvType": recvType, 
            "contentType": contentType, 
            "content": content
         }
        headers = {'Content-Type': 'application/json'}
        return requests.post(self.baseUrl + '/bot/send?token=' + self.token,headers=headers, data=json.dumps(params))

    def batchSendTextMessage(self, recvIds: list, recvType: str, content: map):
        """
        批量，发送文本消息
        """
        return self.batchSendMessage(recvIds, recvType, "text", content)

    def batchSendMarkdownMessage(self, recvIds: list, recvType: str, content: map):
        """
        批量，发送markdown消息
        """
        return self.batchSendMessage(recvIds, recvType, "markdown", content)
    
    def batchSendMessage(self, recvIds: list, recvType: str, contentType: str, content: map):
        """
        批量，批量发送消息
        """
        params = {
            "recvIds": recvIds, 
            "recvType": recvType, 
            "contentType": contentType, 
            "content": content
         }
        headers = {'Content-Type': 'application/json'}
        return requests.post(self.baseUrl + '/bot/batch_send?token=' + self.token,headers=headers, data=json.dumps(params))
    
    def editMessage(self, msgId: str, recvId: str, recvType: str, contentType: str, content: map):
        """
        单条，编辑单条消息
        """
        params = {
            "msgId": msgId, 
            "recvId": recvId, 
            "recvType": recvType, 
            "contentType": contentType, 
            "content": content
         }
        headers = {'Content-Type': 'application/json'}
        return requests.post(self.baseUrl + '/bot/edit?token=' + self.token,headers=headers, data=json.dumps(params))


class Subscription(object):
    """
    订阅事件类
    """
    onMessageNormalSubscriber = None
    onMessageInstructionSubscriber = None
    onGroupJoinSubscriber = None
    onGroupLeaveSubscriber = None
    onBotFollowedSubscriber = None
    onBotUnfollowedSubscriber = None
    onButtonReportInlineSubscriber = None

    def __init__(self) -> None:
        pass

    def listen(self, request):

        eventType = request.get_json()['header']['eventType']
        event = request.get_json()["event"]

        if eventType == 'message.receive.normal': # 普通消息事件  
            self.onMessageNormalSubscriber(event)
        elif eventType == 'message.receive.instruction': # 指令消息事件 
            self.onMessageInstructionSubscriber(event)
        elif eventType == 'group.join': # 加入群组事件 
            self.onGroupJoinSubscriber(event)
        elif eventType == 'group.leave': # 退出群组事件 
            self.onGroupLeaveSubscriber(event)
        elif eventType == 'bot.followed': # 关注机器人事件 
            self.onBotFollowedSubscriber(event)
        elif eventType == 'bot.unfollowed': # 取消关注机器人事件  
            self.onBotUnfollowedSubscriber(event)
        elif eventType == 'button.report.inline': # 消息下按钮点击回调事件 
            self.onButtonReportInlineSubscriber(event)

    def onMessageNormal(self, func):
        self.onMessageNormalSubscriber = func
        return func

    def onMessageInstruction(self, func):
        self.onMessageInstructionSubscriber = func
        return func

    def onGroupJoin(self, func):
        self.onGroupJoinSubscriber = func
        return func

    def onGroupLeave(self, func):
        self.onGroupLeaveSubscriber = func
        return func

    def onBotFollowed(self, func):
        self.onBotFollowedSubscriber = func
        return func

    def onBotUnfollowed(self, func):
        self.onBotUnfollowedSubscriber = func
        return func

    def onButtonReportInline(self, func):
        self.onButtonReportInlineSubscriber = func
        return func


import os
import sys
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))

from flask import Flask, request
import json
import requests
from yunhu.subscription import Subscription
from yunhu.openapi import Openapi


app = Flask(__name__)
sub = Subscription()
openapi = Openapi("xxx")

@app.route('/sub',methods = ['POST'])
def subRoute():
   if request.method == 'POST':
      sub.listen(request)
      return "success"

@sub.onMessageNormal
def onMessageNormalHander(event):
   print("onMessageNormalSubscriber")
   print(event)
   res = openapi.sendMessage(event["sender"]["senderId"], event["sender"]["senderType"], "text", {"text": "机器人回复普通消息"})
   print(res.content)
   
   content = {
      "text": "机器人批量回复普通消息",
      "buttons": [
         {
            "text": "复制内容1",
            "actionType": 2,
            "value": "复制内容1",
         },
         {
            "text": "复制内容2",
            "actionType": 2,
            "value": "复制内容2",
         },
      ],
   }

   res2 = openapi.batchSendMessage([event["sender"]["senderId"]], event["sender"]["senderType"], "text", content)
   print(res2.content)

@sub.onMessageInstruction
def onMessageInstructionHandler(event):
   print("onMessageInstructionlSubscriber")
   print(event)
   res = openapi.editMessage("5e47ce1a65024da2bfba6f17481b6bdd", event["sender"]["senderId"], event["sender"]["senderType"], "text", {"text": "机器人编辑消息"})
   print(res.content)

@sub.onGroupJoin
def onGroupJoinHandler(event):
   print("onGroupJoinSubscriber")
   print(event)

@sub.onGroupLeave
def onGroupLeaveHandler(event):
   print("onGroupLeaveSubscriber")
   print(event)

@sub.onBotFollowed
def onBotFollowedHandler(event):
   print("onBotFollowedSubscriber")
   print(event)

@sub.onBotUnfollowed
def onBotUnfollowedHandler(event):
   print("onBotUnfollowedSubscriber")
   print(event)

@sub.onButtonReportInline
def onButtonReportInlineHandler(event):
   print("onButtonReportInlineSubscriber")
   print(event)





def send_message(params):
   headers = {'Content-Type': 'application/json'}
   r = requests.post('http://chat.jwznb.com:8888/open-apis/v1/bot/send?token=xxx',headers=headers, data=json.dumps(params))


if __name__ == '__main__':
   app.run("0.0.0.0", 8857)
